@@ -16,6 +16,16 @@ DataFrame kmest(const DataFrame data,
                 const std::string conftype,
                 const double conflev);
 
+DataFrame kmdiff(const DataFrame data,
+                 const StringVector& rep,
+                 const StringVector& stratum,
+                 const std::string treat,
+                 const std::string time,
+                 const std::string event,
+                 const double milestone,
+                 const double survDiffH0,
+                 const double conflev);
+
 DataFrame lrtest(const DataFrame data,
                  const StringVector& rep,
                  const StringVector& stratum,
